//CSimpleComboBox.cpp - A simple combobox (i.e. drop down list and arrow button)

#include <stdio.h>
#include "CSimpleComboBox.h"

CSimpleComboBox::CSimpleComboBox(CRect &size,
							     CControlListener *listener,
								 long tag,
							     CColor bgcol,
							     CColor fgcol,
								 int style) :
					 COptionMenu(size,
								 listener,
								 tag,
								 0,
								 0,
								 style)
{
	IsClicked = false;

	BackColour = bgcol;
	ForeColour = fgcol;
	FrameColour = kBlackCColor;

	Scheme = new COptionMenuScheme();

	Scheme->setColors(BackColour, ForeColour, ForeColour, BackColour, ForeColour);
	Scheme->setFont(kNormalFont);

	setScheme(Scheme);

	if(style == kLeft)
		horiTxtAlign = kLeftText;
	else if(style == kRight)
		horiTxtAlign = kRightText;
}

//------------------------------------------------------------------------
void CSimpleComboBox::draw(CDrawContext *pContext)
{
	CRect temp, temp6;
	CPoint temp2[3];
	float temp3, temp4, temp5;

	if(currentIndex >= 0 && nbEntries) 
		drawText(pContext, entry[currentIndex] + prefixNumbers);
	else
		drawText(pContext, NULL);

	temp = size;
	temp.left = temp.right - (temp.bottom - temp.top);

	if(!IsClicked)
	{
		pContext->setFrameColor(FrameColour);
		pContext->drawRect(temp);

		/*pContext->setFillColor(ForeColour);
		//pContext->fillRect(temp);

		temp6 = temp;
		temp6.left += 1;
		temp6.right -= 1;
		temp6.top += 1;
		temp6.bottom -= 1;*/

		pContext->setFillColor(BackColour);
		pContext->fillRect(temp);

		temp3 = (float)(temp.right - temp.left)/3.0f;
		temp4 = (float)(temp.bottom - temp.top)/2.5f;
		temp2[0]((temp.left+(int)temp3), (temp.top+(int)temp4+1));	
		temp2[1]((temp.right-(int)temp3), (temp.top+(int)temp4+1));
		temp5 = ((float)(temp.right-temp.left)*0.5f);
		temp2[2]((temp.left+(int)temp5), (temp.bottom-(int)temp4/*+1*/));
		pContext->setFillColor(ForeColour);
		pContext->fillPolygon(temp2, 3);
		/*pContext->moveTo(temp2[0]);
		temp2[1].h = temp2[1].h + 1;
		pContext->setFrameColor(ForeColour);
		pContext->lineTo(temp2[1]);*/
	}
	else
	{
		pContext->setFrameColor(FrameColour);
		pContext->drawRect(temp);

		pContext->setFillColor(BackColour);
		pContext->fillRect(temp);

		temp6 = temp;
		temp6.left += 1;
		temp6.right -= 1;
		temp6.top += 1;
		temp6.bottom -= 1;

		pContext->setFillColor(ForeColour);
		pContext->fillRect(temp6);

		temp3 = (float)(temp.right - temp.left)/3.0f;
		temp4 = (float)(temp.bottom - temp.top)/2.5f;
		temp2[0]((temp.left+(int)temp3), (temp.top+(int)temp4+1));	
		temp2[1]((temp.right-(int)temp3), (temp.top+(int)temp4+1));
		temp5 = ((float)(temp.right-temp.left)*0.5f);
		temp2[2]((temp.left+(int)temp5), (temp.bottom-(int)temp4/*+1*/));
		pContext->setFillColor(BackColour);
		pContext->fillPolygon(temp2, 3);
		/*pContext->moveTo(temp2[0]);
		temp2[1].h = temp2[1].h + 1;
		pContext->setFrameColor(BackColour);
		pContext->lineTo(temp2[1]);*/
	}
}

//------------------------------------------------------------------------
void CSimpleComboBox::drawText(CDrawContext *pContext, char *string)
{
	CRect temp;

	temp = size;
	temp.right -= (temp.bottom - temp.top);
	if(temp.right < 0)
		temp.right *= -1;

	setDirty(false);

	if(style & kNoDrawStyle)
		return;

	// draw the background
	
	/*if(!bTransparencyEnabled)
	{
		pContext->setFillColor(backColor);
		pContext->fillRect(temp);
	
		if(!(style & (k3DIn|k3DOut))) 
		{
			pContext->setFrameColor(frameColor);
			pContext->drawRect(temp);
		}
	}*/
	pContext->setFrameColor(FrameColour);
	pContext->drawRect(temp);

	pContext->setFillColor(BackColour);
	pContext->fillRect(temp);
	// draw the frame for the 3D effect
	if(style & (k3DIn|k3DOut)) 
	{
		if(style & k3DIn)
			pContext->setFrameColor(backColor);
		else
			pContext->setFrameColor(frameColor);
		CPoint p;
		pContext->moveTo(p(temp.left, temp.bottom));
		pContext->lineTo(p(temp.left, temp.top));
		pContext->lineTo(p(temp.right + 1, temp.top));

		if(style & k3DIn)
			pContext->setFrameColor(frameColor);
		else
			pContext->setFrameColor(backColor);
		pContext->moveTo(p(temp.right, temp.top + 1));
		pContext->lineTo(p(temp.right, temp.bottom));
		pContext->lineTo(p(temp.left, temp.bottom));
	}

	if(!(style & kNoTextStyle) && string)
	{
		pContext->setFont(fontID, 0, txtFace);
	
		// draw darker text (as shadow)
		if(style & kShadowText) 
		{
			CRect newSize(temp);
			newSize.offset(1, 1);
			pContext->setFontColor(shadowColor);
			pContext->drawString(string, newSize, !bTextTransparencyEnabled, horiTxtAlign);
		}
		pContext->setFontColor(ForeColour);
		if(horiTxtAlign == kLeftText)
			temp.x += 3;
		pContext->drawString(string, temp, !bTextTransparencyEnabled, horiTxtAlign);
	}
}

//------------------------------------------------------------------------
void CSimpleComboBox::mouse(CDrawContext *pContext, CPoint &where)
{
	if(!bMouseEnabled || !getParent() || !pContext)
		return;

	lastButton = pContext->getMouseButtons();
	if(lastButton & (kLButton|kRButton|kApple))
	{
		IsClicked = true;
		getParent()->setEditView(this);
		takeFocus(pContext);
	}
}

//------------------------------------------------------------------------
void CSimpleComboBox::takeFocus(CDrawContext *pContext)
{
	if(!getParent())
		return;

	bool multipleCheck = style & (kMultipleCheckStyle & ~kCheckStyle);
	lastResult = -1;
	lastMenu = 0;

	setDirty();

#if WINDOWS
	MSG msg;
	long result = -1;
	HWND hwnd = (HWND)getParent()->getSystemWindow();

	//---Get the position of the Parent
	CRect rect;
	if(pContext)
	{
		rect.left = pContext->offsetScreen.h;
		rect.top  = pContext->offsetScreen.v;
	}
	else
	{
		RECT rctWinParent;
		GetWindowRect(hwnd, &rctWinParent);
		rect.left = rctWinParent.left;
		rect.top  = rctWinParent.top;
	}

	//---Create the popup menu---
	long offIdx = 0;
	appendItems(offIdx);
	
	//---Popup the menu---
	long offset;
	if(style & kPopupStyle)
		offset = (long)(rect.top + size.top);
	else
		offset = (long)(rect.top + size.bottom);

	int flags = TPM_LEFTALIGN;
	if(lastButton & kRButton)
		flags |= TPM_RIGHTBUTTON;

	if(TrackPopupMenu ((HMENU)platformControl, flags, 
			 (int)(rect.left + size.left), offset, 0, hwnd, 0))
	{
		if(PeekMessage(&msg, hwnd, WM_COMMAND, WM_COMMAND, PM_REMOVE))
		{
			if(HIWORD(msg.wParam) == 0)
			{
				result = LOWORD(msg.wParam);
				lastResult = result;
			}
		}
	}

	//---Destroy the menu----
	removeItems();
	
	getParent()->setEditView(0);

	//---Update the dependencies
	if(result != -1 || bgWhenClick)
	{
		CDrawContext *pContextTemp = 0;
		HDC hdc;
		if(!pContext && result != -1)
		{
	 		// create a local context
			hdc = GetDC(hwnd);
			pContextTemp = new CDrawContext(getParent(), hdc, hwnd);
		}
		else
			pContextTemp = pContext;
	
		// to force the redraw
		//if(bgWhenClick)
			setDirty();

		if(result != -1)
		{
			long idx = 0;
			offIdx = 0;
			COptionMenu *menu = getItemMenu(result, idx, offIdx);
			if(menu)
			{
				lastMenu = menu;
				menu->setValue((float)idx);
			
				// update dependency
				if(listener)
					listener->valueChanged(pContextTemp, menu);
			}
		}

		// redraw the display
		// AAAAARRRRGHHHHHHHHHHHHH!!
		doIdleStuff ();
		setDirty (false);

		if(!pContext && pContextTemp)
		{
			delete pContextTemp;
			ReleaseDC(hwnd, hdc);
		}
	}

#elif MAC
	// no entries, no menu
	if(nbEntries == 0)
	{
		getParent()->setEditView(0);
		return;
	}
	
	//---Get the position of the Parent
	WindowPtr theWindow = (WindowPtr)getParent()->getSystemWindow();
	
	//---Transform local coordinates to global coordinates
	long offset;
	if(style & kPopupStyle)
		offset = (int)size.top;
	else
		offset = (int)size.bottom;
	
	Rect bounds;
	GetPortBounds(GetWindowPort(theWindow), &bounds);
	Point LToG;
	LToG.v = bounds.top + offset;
	LToG.h = bounds.left + size.left;
	
	if(pContext)
	{
		LToG.h += pContext->offset.h;
		LToG.v += pContext->offset.v;
	}
	
	LocalToGlobal(&LToG);
	
	//---Create the popup menu---
	long offIdx = 0;
	MenuHandle theMenu = (MenuHandle)appendItems(offIdx);
		
	// Calculate the menu size (height and width)
	CalcMenuSize(theMenu);
	
	// Get a handle to the screen
	RgnHandle rgn = GetGrayRgn();
	#if CARBON
	GetRegionBounds(rgn, &bounds);
	int bottom      = bounds.bottom;
	long menuHeight = GetMenuHeight(theMenu);
	#else
	int bottom      = (*rgn)->rgnBBox.bottom;
	long menuHeight = (*theMenu)->menuHeight;
	#endif

	// Calculate the size of one menu item (round to the next int)
	int menuItemSize = (menuHeight + nbEntries - 1) / nbEntries;
	
	//---Popup the Menu
	long popUpItem = 1;
	long PopUpMenuItem = 0;
	if(LToG.v + menuHeight >= bottom - menuItemSize / 2)
	{
		if(nbEntries * menuItemSize >= bottom)
		{
			popUpItem = currentIndex + 2;
			if(popUpItem > nbEntries)
				popUpItem = nbEntries;
		}
		if(nbEntries * menuItemSize >= bottom)
			PopUpMenuItem = PopUpMenuSelect(theMenu, LToG.v, LToG.h, popUpItem);
		else
			PopUpMenuItem = PopUpMenuSelect(theMenu, bottom - menuHeight - menuItemSize, LToG.h, popUpItem);
	}
	else
		PopUpMenuItem = PopUpMenuSelect(theMenu, LToG.v, LToG.h, popUpItem);
		
	//---Destroy the menu----
	removeItems();
	
	// HiWord indicates MenuID, LoWord indicates the item index
	short result = LoWord(PopUpMenuItem) - 1;	
	lastResult = result;
	short menuIDResult = HiWord(PopUpMenuItem);
	if(menuIDResult != 0 || bgWhenClick) 
	{
		// to force the redraw
		if(bgWhenClick)
			setDirty();

		CDrawContext *pContextTemp = 0;
		if(!pContext && menuIDResult != 0)
		{
			// create a local context
			pContextTemp = new CDrawContext(getParent(), 0, theWindow);
		}
		else
			pContextTemp = pContext;
		if(menuIDResult != 0)
		{
			long idx = 0;
			offIdx = menuIDResult;
			COptionMenu *menu = getItemMenu(result, idx, offIdx);
			if(menu)
			{
				lastMenu = menu;
				menu->setValue(result);
				if(listener)
					listener->valueChanged(pContextTemp, menu);
			}
		}

		// redraw the display
		doIdleStuff();
		setDirty(false);

		if(!pContext && pContextTemp)
			delete pContextTemp;
	}
	if(getParent()->getEditView() == this)
		getParent()->setEditView(0);

#elif MOTIF
	Arg args[10];
	int n = 0;
	
	// get the position of the pParent
	CRect rect;
	getParent()->getSize(&rect);

	if(pContext)
	{
		rect.left += pContext->offset.h;
		rect.top  += pContext->offset.v;
	}

	// create a popup menu
	int offset;
	if(style & kPopupStyle)
		offset = (int)(rect.top + size.top);
	else
		offset = (int)(rect.top + size.bottom);

	XtSetArg(args[n], XmNx, rect.left + size.left); n++;
	XtSetArg(args[n], XmNy, offset); n++;
	XtSetArg(args[n], XmNmenuHistory, currentIndex); n++;
	XtSetArg(args[n], XmNtraversalOn, true); n++;

	platformControl = (void*)XmCreatePopupMenu ((Widget)(getParent()->getSystemWindow()), 
			"popup", args, n);

	XtAddCallback((Widget)platformControl, XmNunmapCallback, _unmapCallback, this);

	// insert the menu items
	for(long i=0;i<nbEntries;i++)
	{
		if(!strcmp(entry[i], kMenuSeparator))
		{
			itemWidget[i] = (void*)XtCreateManagedWidget("separator", 
							 xmSeparatorGadgetClass, (Widget)platformControl, 0, 0);
		}
		else
		{
			if(multipleCheck)
			{
				itemWidget[i] = (void*)XtVaCreateManagedWidget(entry[i], 
					xmToggleButtonWidgetClass, (Widget)platformControl,
					XmNset, check[i], XmNvisibleWhenOff, false, 0);
				XtAddCallback((Widget)itemWidget[i], XmNvalueChangedCallback, _activateCallback, this);
			}
			else if(style & kCheckStyle)
			{
				itemWidget[i] = (void*)XtVaCreateManagedWidget(entry[i], 
					xmToggleButtonWidgetClass, (Widget)platformControl,
					XmNset, (i == currentIndex) ? true : false, XmNvisibleWhenOff, false, 0);
				XtAddCallback((Widget)itemWidget[i], XmNvalueChangedCallback, _activateCallback, this);
			}
			else 
			{
				itemWidget[i] = (void*)XtVaCreateManagedWidget(entry[i], 
					xmPushButtonWidgetClass, (Widget)platformControl, 0);
				XtAddCallback((Widget)itemWidget[i], XmNactivateCallback, _activateCallback, this);
			}
		}
	}

	XtManageChild((Widget)platformControl);

#elif BEOS
	BPopUpMenu* popup = new BPopUpMenu("popup", false, false);
	BMessage*	message;
	BMenuItem*	item;
	for(long i=0;i<nbEntries;i++)
	{	
		if(strcmp(entry[i], kMenuSeparator) == 0)
			popup->AddSeparatorItem();
		else
		{
			message = new BMessage(i);
			item = new BMenuItem(entry[i], message);
			popup->AddItem(item);
			if(multipleCheck)
			{
				if(check[i])
					item->SetMarked(true);
			}
			else if((style & kCheckStyle) && currentIndex == i)
				item->SetMarked(true);
		}
	}
	CRect rect;
	getParent()->getSize(&rect);
	if(pContext)
	{
		rect.left += pContext->offset.h;
		rect.top  += pContext->offset.v;
	}
	long offset;
	if(style & kPopupStyle)
		offset = (int)(rect.top + size.top);
	else
		offset = (int)(rect.top + size.bottom);
	BPoint where(rect.left + size.left, offset);
	BView* plugView = (BView*)getParent()->getSystemWindow();
	plugView->ConvertToScreen(&where);
	item = popup->Go(where);
	if(item)
	{
		message = item->Message();
		if(message)
		{
			CDrawContext *pContextTemp = pContext;
			// create a local context
			if(!pContextTemp)
				pContextTemp = new CDrawContext(getParent(), plugView, NULL);

			setValue(message->what);
			
			if(listener)
				listener->valueChanged(pContextTemp, this);
	
			// redraw the display
			draw(pContextTemp);

			if(!pContext && pContextTemp)
				delete pContextTemp;
		}
	}
	delete popup;
	getParent()->setEditView(0);
#endif

	IsClicked = false;
	setDirty();
}

//------------------------------------------------------------------------
void CSimpleComboBox::looseFocus(CDrawContext *pContext)
{
	COptionMenu::looseFocus(pContext);
	IsClicked = false;
	setDirty();
}
