//CSimpleKeyboard.cpp - Keyboard base class: just draws the keys, basically

#include <math.h>
#include "CSimpleKeyboard.h"

//-----------------------------------------------------------------------
float round(float val)	//unless I'm mistaken, there isn't a rounding function for C/C++
{
	float temp;

	temp = (float)fmod(val, (float)((int)val));

	if(temp < 0.5f)
		return (float)((int)val);
	else
		return (float)((int)val+1);
}

//-----------------------------------------------------------------------------
CSimpleKeyboard::CSimpleKeyboard(CRect &size,
							     CControlListener *listener,
								 CColor bgcol,
								 CColor fgcol,
								 int numoctaves):
						CControl(size,
							     listener)
{
	NumOctaves = numoctaves;

	BackColour = bgcol;
	ForeColour = fgcol;
	FrameColour = kBlackCColor;
}

/*//-----------------------------------------------------------------------------
bool CSimpleKeyboard::attached (CView *parent)
{
	if (OS)
		delete OS;

	OS = new COffscreenContext(getParent(), (size.right-size.left), (size.bottom-size.top), kBlackCColor);
		
	return CControl::attached(parent);
}

//-----------------------------------------------------------------------------
bool CSimpleKeyboard::removed (CView *parent)
{
	if (OS)
	{
		delete OS;
		OS = 0;
	}
	return CControl::removed (parent);
}*/

//-----------------------------------------------------------------------------
void CSimpleKeyboard::draw(CDrawContext *pContext)
{
	bool temp[127];

	COffscreenContext *oc = new COffscreenContext(getParent(), (size.right-size.left), (size.bottom-size.top), kBlackCColor);

	for(int i=0;i<127;i++)
		temp[i] = false;

	if(oc)
	{
		DrawKeyboard(oc, temp);
		oc->copyFrom(pContext, size);
	}

	delete oc;
}

//-----------------------------------------------------------------------------
void CSimpleKeyboard::DrawKeyboard(CDrawContext *oc, bool *keys)
{
	int numwhitekeys, numblackkeys;
	int temp1, temp2, temp3;
	int i, j;
	bool onthree = false;
	int blackcount;
	int current_octave = 0;

	CRect rect(0, 0, (size.right-size.left), ((size.bottom-1)-size.top));
	oc->setFillColor(kWhiteCColor);
	oc->fillRect(rect);
	oc->setFrameColor(kBlackCColor);
	rect.right--;
	oc->drawRect(rect);

	if(NumOctaves > 10)
	{
		numwhitekeys = 74;
		numblackkeys = 53;
	}
	else
	{
		numwhitekeys = (int)round(7.0f * (float)NumOctaves);
		numblackkeys = (int)round(5.0f * (float)NumOctaves);
	}

	temp1 = (int)round(((float)size.right-(float)size.left)/(float)numwhitekeys);

	oc->setFrameColor(FrameColour);

	rect(0, 0, temp1, ((size.bottom-1)-size.top));

	for(i=0;i<numwhitekeys;i++)
	{
		if(keys[i])
			oc->setFillColor(BackColour);
		else
			oc->setFillColor(kWhiteCColor);
		oc->drawRect(rect);
		oc->fillRect(rect);

		rect.left += temp1;
		rect.right += temp1;
	}

	temp3 = (int)round(0.65f * (float)temp1);
	temp2 = (int)round(0.65f * (((float)size.bottom-1.0f)-(float)size.top));

	rect(0, 0, temp3, temp2);
	rect.left += temp1-(temp3/2);
	rect.right = rect.left + temp3;

	//rect.offset(size.left, size.top);

	//OS->fillRect(rect);
	//OS->drawRect(rect);

	j = numwhitekeys;
	blackcount = 0;
	for(i=0;i<numwhitekeys;i++)
	{
		if((!onthree)&&(blackcount < 2))
		{
			if(keys[j])
				oc->setFillColor(ForeColour);
			else
				oc->setFillColor(kBlackCColor);

			oc->fillRect(rect);
			oc->drawRect(rect);

			rect.left += temp1;
			rect.right = rect.left + temp3;

			blackcount++;
			j++;
		}
		else if((!onthree)&&(blackcount < 3))
		{
			rect.left += temp1;
			rect.right = rect.left + temp3;

			blackcount = 0;
			onthree = true;
		}
		else if((onthree)&&(blackcount < 3))
		{

			if(keys[j])
				oc->setFillColor(ForeColour);
			else
				oc->setFillColor(kBlackCColor);

			oc->fillRect(rect);
			oc->drawRect(rect);

			rect.left += temp1;
			rect.right = rect.left + temp3;

			blackcount++;
			j++;
		}
		else
		{
			rect.left += temp1;
			rect.right = rect.left + temp3;

			blackcount = 0;
			onthree = false;
		}
	}
	
}