//CSimpleKeyboardSlider.cpp - A keyboard widget which displays a keyboard with a single key selected (and acts like a slider)

#include "CSimpleKeyboardSlider.h"

//-----------------------------------------------------------------------
CSimpleKeyboardSlider::CSimpleKeyboardSlider(CRect &size,
										     CControlListener *listener,
										     long tag,
										     CColor bgcol,
										     CColor fgcol,
											 int numoctaves):
							 CSimpleKeyboard(size,
											 listener,
											 bgcol,
											 fgcol,
											 numoctaves)
{
	setTag(tag);
	if(numoctaves > 10)
		NumKeys = 127;
	else
		NumKeys = numoctaves * 12;

	TempSelectedKey = 0;
}

void CSimpleKeyboardSlider::setValue(float val)
{
	int newkey, oldkey;

	oldkey = ConvertToActualKey((int)(value*(float)NumKeys));
	newkey = ConvertToActualKey((int)(val*(float)NumKeys));

	if(TempSelectedKey)
	{
		TempSelectedKey[oldkey] = false;
		if(newkey >= 127)
			TempSelectedKey[NumKeys-1] = true;
		else if(newkey < 0)
			TempSelectedKey[0] = true;
		else
			TempSelectedKey[newkey] = true;
	}

	value = val;
}

//-----------------------------------------------------------------------------
bool CSimpleKeyboardSlider::attached (CView *parent)
{
	if(!TempSelectedKey)
	{
		TempSelectedKey = new bool[NumKeys];
		for(int i=0;i<NumKeys;i++)
			TempSelectedKey[i] = false;
		setValue(value);
	}

	return CView::attached(parent);
}

//-----------------------------------------------------------------------------
bool CSimpleKeyboardSlider::removed (CView *parent)
{
	if(TempSelectedKey)
	{
		delete [] TempSelectedKey;
		TempSelectedKey = 0;
	}
	return CSimpleKeyboard::removed(parent);
}

//-----------------------------------------------------------------------------
void CSimpleKeyboardSlider::draw(CDrawContext *pContext)
{
	COffscreenContext *oc = new COffscreenContext(getParent(), getWidth(), getHeight(), kBlackCColor);

	if(TempSelectedKey)
	{
		DrawKeyboard(oc, TempSelectedKey);
		oc->copyFrom(pContext, size);
	}

	delete oc;
}

//-----------------------------------------------------------------------------
void CSimpleKeyboardSlider::mouse(CDrawContext *pContext, CPoint &where)
{
	if(!bMouseEnabled)
		return;

	long button = pContext->getMouseButtons();

	// set the default value
	if((button == (kControl|kLButton))||(button == kRButton))
	{
		value = getDefaultValue();
		if(isDirty() && listener)
			listener->valueChanged(pContext, this);
		return;
	}
	
	// allow left mousebutton only
	if(!(button & kLButton))
		return;
	
	float fValue = value;

	int where_we_are;	//where the gauge is currently set at

	//if(style & kHorizontal)
	//{
		where_we_are = (size.left+2) + (int)(fValue * (float)((size.right-2)-(size.left+2)));
	/*}
	else
	{
		where_we_are = (size.top+2) + (int)(fValue * (float)((size.bottom-2)-(size.top+2)));
	}*/

	float oldVal    = value;
	long  oldButton = button;

	getParent()->beginEdit(tag);
	getParent()->setEditView(this);

	while(1)
	{
		button = pContext->getMouseButtons();
		if(!(button & kLButton))
			break;

		if((oldButton != button)&&(button & kShift))
		{
			oldVal    = value;
			oldButton = button;
		}
		else if(!(button & kShift))
			oldVal = value;

		/*if(style & kHorizontal)
			value = (float)(where.h - where_we_are)/(float)rangeHandle;
		else
			value = (float)(where.v - where_we_are)/(float)rangeHandle;*/

		//if(style & kHorizontal)
			where_we_are = where.h;
		/*else
			where_we_are = where.v;*/

		//if(style & kHorizontal)
			fValue = (float)(where_we_are-(size.left+2))/(float)((size.right-2)-(size.left+2));
		/*else
			fValue = (float)(where_we_are-(size.top+2))/(float)((size.bottom-2)-(size.top+2));*/
			
		/*if((style & kRight)||(style & kBottom))
			//value = 1.0f-fValue;
			setValue(1.0f-fValue);
		else*/
			//value = fValue;
			setValue(fValue);

		/*if(button & kShift)
			//value = oldVal + ((value - oldVal)/zoomFactor);
			setValue(oldVal+((value-oldVal)/zoomFactor));*/
		bounceValue();

		if(isDirty() && listener)
		{
			listener->valueChanged(pContext, this);
		}

		pContext->getMouseLocation(where);

		doIdleStuff();
	}

	// end of edit parameter
	getParent()->endEdit(tag);
}

//-----------------------------------------------------------------------------
int CSimpleKeyboardSlider::ConvertToActualKey(int keyno)
{
	int numwhitekeys;
	int numblackkeys;
	int j = 0;
	bool IsBlack;
	int whitecount = -1;
	int blackcount = -1;

	if(NumOctaves > 10)
	{
		numwhitekeys = 74;
		numblackkeys = 53;
	}
	else
	{
		numwhitekeys = (int)round(7.0f * (float)NumOctaves);
		numblackkeys = (int)round(5.0f * (float)NumOctaves);
	}

	for(int i=0;i<NumKeys;i++)
	{
		switch(j)
		{
			case 0:
				IsBlack = false;
				whitecount++;
				break;
			case 1:
				IsBlack = true;
				blackcount++;
				break;
			case 2:
				IsBlack = false;
				whitecount++;
				break;
			case 3:
				IsBlack = true;
				blackcount++;
				break;
			case 4:
				IsBlack = false;
				whitecount++;
				break;
			case 5:
				IsBlack = false;
				whitecount++;
				break;
			case 6:
				IsBlack = true;
				blackcount++;
				break;
			case 7:
				IsBlack = false;
				whitecount++;
				break;
			case 8:
				IsBlack = true;
				blackcount++;
				break;
			case 9:
				IsBlack = false;
				whitecount++;
				break;
			case 10:
				IsBlack = true;
				blackcount++;
				break;
			case 11:
				IsBlack = false;
				whitecount++;
				j = -1;
				break;
		}
		j++;
		if(i==keyno)
			break;
	}
	if(IsBlack)
		return numwhitekeys + blackcount;
	else
		return whitecount;
}