//CSimpleKeyboardSliderBP.cpp - A keyboard widget made to display an editable selected range of keys

#include <math.h>
#include "CSimpleKeyboardSliderBP.h"

CSimpleKeyboardSliderBP::CSimpleKeyboardSliderBP(CRect &size,
						    CControlListener *listener,
						    long tag1,
							long tag2,
						    CColor bgcol,
						    CColor fgcol,
							int numoctaves):
	  CSimpleKeyboardSlider(size,
							listener,
							tag1,
							bgcol,
							fgcol,
							numoctaves)
{
	Tag1 = tag1;
	Tag2 = tag2;

	SetValue1(0.0f);
	SetValue2(1.0f);
}

void CSimpleKeyboardSliderBP::SetValue1(float val)
{
	int newkey, oldkey, v2key, tempkey;
	int i;

	oldkey = (int)round(Value1*(float)NumKeys);
	newkey = (int)round(val*(float)NumKeys);
	v2key = (int)round(Value2*(float)NumKeys);

	if(TempSelectedKey)
	{
		for(i=0;i<NumKeys;i++)
			TempSelectedKey[i] = false;

		if(newkey < v2key)
		{
			for(i=newkey;i<=v2key;i++)
			{
				tempkey = ConvertToActualKey(i);
				TempSelectedKey[tempkey] = true;
			}
		}
		else
		{
			for(i=v2key;i<=newkey;i++)
			{
				tempkey = ConvertToActualKey(i);
				TempSelectedKey[tempkey] = true;
			}
		}
	}

	Value1 = val;
	setDirty();
}

void CSimpleKeyboardSliderBP::SetValue2(float val)
{
	int newkey, oldkey, v1key, tempkey;
	int i;

	oldkey = (int)round(Value2*(float)NumKeys);
	newkey = (int)round(val*(float)NumKeys);
	v1key = (int)round(Value1*(float)NumKeys);

	if(TempSelectedKey)
	{
		for(i=0;i<NumKeys;i++)
			TempSelectedKey[i] = false;

		if(newkey < v1key)
		{
			for(i=newkey;i<=v1key;i++)
			{
				tempkey = ConvertToActualKey(i);
				TempSelectedKey[tempkey] = true;
			}
		}
		else
		{
			for(i=v1key;i<=newkey;i++)
			{
				tempkey = ConvertToActualKey(i);
				TempSelectedKey[tempkey] = true;
			}
		}
	}

	Value2 = val;
	setDirty();
}

//-----------------------------------------------------------------------------
bool CSimpleKeyboardSliderBP::attached (CView *parent)
{
	if(!TempSelectedKey)
	{
		TempSelectedKey = new bool[NumKeys];
		for(int i=0;i<NumKeys;i++)
			TempSelectedKey[i] = false;
		SetValue1(Value1);
		SetValue2(Value2);
	}

	return CSimpleKeyboard::attached(parent);
}

//-----------------------------------------------------------------------------
void CSimpleKeyboardSliderBP::mouse(CDrawContext *pContext, CPoint &where)
{
	CRect rect, rect2;
	float oldVal;
	long  oldButton;
	int where_we_are_1, where_we_are_2;
	int which_end = 0;
	float fValue;
	int temp_middle;

	if(!bMouseEnabled)
		return;

	long button = pContext->getMouseButtons();
	
	// allow left mousebutton only
	if(!(button & kLButton))
		return;

	where_we_are_1 = (size.left+2) + (int)(Value1 * (float)((size.right-2)-(size.left+2)));
	where_we_are_2 = (size.left+2) + (int)(Value2 * (float)((size.right-2)-(size.left+2)));

	oldVal    = value;
	oldButton = button;

	getParent()->beginEdit(tag);
	getParent()->setEditView(this);

	if(where_we_are_1 < where_we_are_2)
	{
		temp_middle = (where_we_are_1 - where_we_are_2)/2;
		temp_middle += where_we_are_2;

		if(where.h > temp_middle)
		{
			which_end = 1;
			setTag(Tag2);
		}
		else
		{
			which_end = 0;
			setTag(Tag1);
		}
	}
	else
	{
		temp_middle = (where_we_are_2 - where_we_are_1)/2;
		temp_middle += where_we_are_1;

		if(where.h < temp_middle)
		{
			which_end = 1;
			setTag(Tag2);
		}
		else
		{
			which_end = 0;
			setTag(Tag1);
		}
	}

	while(1)
	{
		button = pContext->getMouseButtons();
		if(!(button & kLButton))
			break;

		if((oldButton != button)&&(button & kShift))
		{
			if(which_end == 0)
				oldVal = Value1;
			else
				oldVal = Value2;
			oldButton = button;
		}
		else if(!(button & kShift))
		{
			if(which_end == 0)
				oldVal = Value1;
			else
				oldVal = Value2;
		}

		if(which_end == 0)
		{
			where_we_are_1 = where.h;
			fValue = (float)(where_we_are_1-(size.left+2))/(float)((size.right-2)-(size.left+2));
		}
		else
		{
			where_we_are_2 = where.h;
			fValue = (float)(where_we_are_2-(size.left+2))/(float)((size.right-2)-(size.left+2));
		}
		if(fValue > 1.0f)
			fValue = 1.0f;
		if(fValue < 0.0f)
			fValue = 0.0f;

		if(which_end == 0)
			SetValue1(fValue);
		else
			SetValue2(fValue);

		if(listener)
		{
			setDirty();
			listener->valueChanged(pContext, this);
		}

		pContext->getMouseLocation(where);

		doIdleStuff();
	}

	// end of edit parameter
	getParent()->endEdit(tag);
}